<?php namespace MasterPopups\Includes;

class Settings
{
    public $plugin = null;
    public static $options = array();
    public $xbox = null;
    private static $instance = null;

    private function __construct($plugin = null)
    {
        $this->plugin = $plugin;
        $this->xbox = xbox_get($this->plugin->options_manager->mb_settings);
        $this->set_options();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone()
    {
    }//Stopping Clonning of Object

    private function __wakeup()
    {
    }//Stopping unserialize of object

    public static function init($plugin = null)
    {
        if (null === self::$instance) {
            self::$instance = new self($plugin);
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Acceso a cualquier opción
    |---------------------------------------------------------------------------------------------------
    */
    public static function option($option_name = '', $default_value = null)
    {
        if (isset(self::$options[$option_name])) {
            return self::$options[$option_name];
        } else if ($default_value) {
            self::$options[$option_name] = $default_value;
            return self::$options[$option_name];
        }
        return null;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Establece las opciones por defecto
    |---------------------------------------------------------------------------------------------------
    */
    private function set_options($options = array())
    {
        $default_options = array(
            'mx-record-email-validation' => $this->xbox->get_field_value('mx-record-email-validation', 'on'),
            'custom-css' => $this->xbox->get_field_value('custom-css', ''),
            'custom-javascript' => $this->xbox->get_field_value('custom-javascript', ''),
            'integrated-services' => $this->xbox->get_field_value('integrated-services', array()),
            'load-google-fonts' => $this->xbox->get_field_value('load-google-fonts', 'on'),
            'load-font-awesome' => $this->xbox->get_field_value('load-font-awesome', 'on'),
            'send-data-to-developer' => $this->xbox->get_field_value('send-data-to-developer', 'on'),
            'popups-z-index' => $this->xbox->get_field_value('popups-z-index', '99999999'),
            //'cookies' => $this->xbox->get_field_value('cookies', array()),
        );

        self::$options = wp_parse_args($options, $default_options);
        self::$options = wp_parse_args(self::$options, $this->public_options());

        return self::$options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Opciones que serán públicas
    |---------------------------------------------------------------------------------------------------
    */
    public static function public_options($options = array())
    {
        return array(
            'is_admin' => is_admin(),
            'integrated_services' => self::get_status_integrated_services(),
            'popups_z_index' => self::$options['popups-z-index'],
            //'custom_cookies' => self::get_custom_cookies(),
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Retorna los servicios integrados con su estado conectado o desconectado
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_status_integrated_services()
    {
        $value = (array)self::option('integrated-services');
        $integrated_services = array();
        foreach ($value as $index => $service) {
            $integrated_services[$service['integrated-services_type']] = $service['service-status'];
        }
        return $integrated_services;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Retorna las cookies creadas por el usuario
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_custom_cookies()
    {
//        $saved_cookies = self::option('cookies');
//        $cookies = array();
//        if (is_array($saved_cookies)) {
//            foreach ($saved_cookies as $key => $value) {
//                unset($saved_cookies[$key]['cookies_type'],
//                    $saved_cookies[$key]['cookies_visibility'],
//                    $saved_cookies[$key]['cookies_name']);
//
//                if (!isset($saved_cookies[$key]['behavior'])) {
//                    $saved_cookies[$key]['behavior'] = array();
//                }
//                $cookies[$value['name']] = $saved_cookies[$key];
//            }
//        }
//        return $cookies;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Activación del Plugin
    |---------------------------------------------------------------------------------------------------
    */
    public static function plugin_status()
    {
        $option = get_option('mpp-plugin-status');
        if ($option && is_array($option)) {
            return isset($option['purchase_code'], $option['user_name']);
        }
        return false;
    }

    public static function plugin_status_message($url)
    {
        return '<div class="ampp-message ampp-message-warning ampp-icon-message">
			<header>' . __('License activation required.', 'masterpopups') . '</header>
			<p>' . sprintf(__('Please activate your license from %shere%s. Tab "Plugin Activation"', 'masterpopups'), '<a href="' . $url . '" target="_blank">', '</a>') . '</p>
	</div>';
    }


}
